@@ -127,7 +127,6 @@ static void
 udisks_client_finalize (GObject *object)
 {
   UDisksClient *client = UDISKS_CLIENT (object);
-  UDisksClientClass *client_class = UDISKS_CLIENT_GET_CLASS (client);
 
   if (client->changed_timeout_source != NULL)
     g_source_destroy (client->changed_timeout_source);
@@ -135,12 +134,6 @@ udisks_client_finalize (GObject *object)
   if (client->initialization_error != NULL)
     g_clear_error (&(client->initialization_error));
 
-  if (client_class->changed_blacklist != NULL)
-    {
-      g_hash_table_destroy (client_class->changed_blacklist);
-      client_class->changed_blacklist = NULL;
-    }
-
   /* might be NULL if failing early in the constructor */
   if (client->object_manager != NULL)
     {
@@ -861,6 +854,66 @@ udisks_client_get_mdraid_for_block (UDisksClient  *client,
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+typedef const gchar *
+(*member_retrieve_cb) (UDisksBlock *object);
+
+static GList *
+_udisks_client_get_block_or_blocks_for_mdraid (UDisksClient *client,
+                                               UDisksMDRaid *raid,
+                                               member_retrieve_cb member_get,
+                                               gboolean only_first_one,
+                                               gboolean skip_partitions)
+{
+  GList *ret = NULL;
+  GList *l, *object_proxies = NULL;
+  GDBusObject *raid_object;
+  const gchar *raid_objpath;
+
+  g_return_val_if_fail (UDISKS_IS_CLIENT (client), NULL);
+  g_return_val_if_fail (UDISKS_IS_MDRAID (raid), NULL);
+
+  raid_object = g_dbus_interface_get_object (G_DBUS_INTERFACE (raid));
+  if (raid_object == NULL)
+    goto out;
+
+  raid_objpath = g_dbus_object_get_object_path (raid_object);
+
+  object_proxies = g_dbus_object_manager_get_objects (client->object_manager);
+  for (l = object_proxies; l != NULL; l = l->next)
+    {
+      UDisksObject *object = UDISKS_OBJECT (l->data);
+      UDisksBlock *block;
+
+      block = udisks_object_get_block (object);
+      if (block == NULL)
+        continue;
+
+      /* ignore partitions */
+      if (skip_partitions)
+        {
+          if (udisks_object_peek_partition (object) != NULL)
+            continue;
+        }
+
+      if (g_strcmp0 (member_get (block), raid_objpath) == 0)
+        {
+          ret = g_list_prepend (ret, block);
+
+          if (only_first_one)
+            goto out;
+        }
+      else
+        {
+          g_object_unref (block);
+        }
+    }
+
+ out:
+  g_list_foreach (object_proxies, (GFunc) g_object_unref, NULL);
+  g_list_free (object_proxies);
+  return ret;
+}
+
 /**
  * udisks_client_get_block_for_mdraid:
  * @client: A #UDisksClient.
@@ -886,45 +939,19 @@ UDisksBlock *
 udisks_client_get_block_for_mdraid (UDisksClient *client,
                                     UDisksMDRaid *raid)
 {
+  GList *b_list = NULL;
   UDisksBlock *ret = NULL;
-  GList *l, *object_proxies = NULL;
-  GDBusObject *raid_object;
-  const gchar *raid_objpath;
 
-  g_return_val_if_fail (UDISKS_IS_CLIENT (client), NULL);
-  g_return_val_if_fail (UDISKS_IS_MDRAID (raid), NULL);
-
-  raid_object = g_dbus_interface_get_object (G_DBUS_INTERFACE (raid));
-  if (raid_object == NULL)
-    goto out;
-
-  raid_objpath = g_dbus_object_get_object_path (raid_object);
-
-  object_proxies = g_dbus_object_manager_get_objects (client->object_manager);
-  for (l = object_proxies; l != NULL; l = l->next)
-    {
-      UDisksObject *object = UDISKS_OBJECT (l->data);
-      UDisksBlock *block;
-
-      block = udisks_object_get_block (object);
-      if (block == NULL)
-        continue;
-
-      /* ignore partitions */
-      if (udisks_object_peek_partition (object) != NULL)
-        continue;
-
-      if (g_strcmp0 (udisks_block_get_mdraid (block), raid_objpath) == 0)
-        {
-          ret = block;
-          goto out;
-        }
-      g_object_unref (block);
-    }
-
- out:
-  g_list_foreach (object_proxies, (GFunc) g_object_unref, NULL);
-  g_list_free (object_proxies);
+  b_list = _udisks_client_get_block_or_blocks_for_mdraid (client,
+                                                          raid,
+                                                          udisks_block_get_mdraid,
+                                                          TRUE,   /* Retrieve first one */
+                                                          TRUE);  /* Skip partitions */
+  if (b_list)
+   {
+      ret = (g_list_first (b_list))->data;
+      g_list_free (b_list);
+   }
   return ret;
 }
 
@@ -951,49 +978,11 @@ GList *
 udisks_client_get_all_blocks_for_mdraid (UDisksClient *client,
                                          UDisksMDRaid *raid)
 {
-  GList *ret = NULL;
-  GList *l, *object_proxies = NULL;
-  GDBusObject *raid_object;
-  const gchar *raid_objpath;
-
-  g_return_val_if_fail (UDISKS_IS_CLIENT (client), NULL);
-  g_return_val_if_fail (UDISKS_IS_MDRAID (raid), NULL);
-
-  raid_object = g_dbus_interface_get_object (G_DBUS_INTERFACE (raid));
-  if (raid_object == NULL)
-    goto out;
-
-  raid_objpath = g_dbus_object_get_object_path (raid_object);
-
-  object_proxies = g_dbus_object_manager_get_objects (client->object_manager);
-  for (l = object_proxies; l != NULL; l = l->next)
-    {
-      UDisksObject *object = UDISKS_OBJECT (l->data);
-      UDisksBlock *block;
-
-      block = udisks_object_get_block (object);
-      if (block == NULL)
-        continue;
-
-      /* ignore partitions */
-      if (udisks_object_peek_partition (object) != NULL)
-        continue;
-
-      if (g_strcmp0 (udisks_block_get_mdraid (block), raid_objpath) == 0)
-        {
-          ret = g_list_prepend (ret, block);
-        }
-      else
-        {
-          g_object_unref (block);
-        }
-    }
-
- out:
-  g_list_foreach (object_proxies, (GFunc) g_object_unref, NULL);
-  g_list_free (object_proxies);
-  ret = g_list_reverse (ret);
-  return ret;
+  return g_list_reverse (_udisks_client_get_block_or_blocks_for_mdraid (client,
+                                                                        raid,
+                                                                        udisks_block_get_mdraid,
+                                                                        FALSE,    /* Retrieve all */
+                                                                        TRUE));   /* Skip partitions */
 }
 
 /**
@@ -1013,46 +1002,12 @@ GList *
 udisks_client_get_members_for_mdraid (UDisksClient *client,
                                       UDisksMDRaid *raid)
 {
-  GList *ret = NULL;
-  GList *l, *object_proxies = NULL;
-  GDBusObject *raid_object;
-  const gchar *raid_objpath;
-
-  g_return_val_if_fail (UDISKS_IS_CLIENT (client), NULL);
-  g_return_val_if_fail (UDISKS_IS_MDRAID (raid), NULL);
-
-  raid_object = g_dbus_interface_get_object (G_DBUS_INTERFACE (raid));
-  if (raid_object == NULL)
-    goto out;
-
-  raid_objpath = g_dbus_object_get_object_path (raid_object);
-
-  object_proxies = g_dbus_object_manager_get_objects (client->object_manager);
-  for (l = object_proxies; l != NULL; l = l->next)
-    {
-      UDisksObject *object = UDISKS_OBJECT (l->data);
-      UDisksBlock *block;
-
-      block = udisks_object_get_block (object);
-      if (block == NULL)
-        continue;
-
-      if (g_strcmp0 (udisks_block_get_mdraid_member (block), raid_objpath) == 0)
-        {
-          ret = g_list_prepend (ret, block); /* adopts reference to block */
-        }
-      else
-        {
-          g_object_unref (block);
-        }
-    }
-
- out:
-  g_list_foreach (object_proxies, (GFunc) g_object_unref, NULL);
-  g_list_free (object_proxies);
-  return ret;
+  return _udisks_client_get_block_or_blocks_for_mdraid (client,
+                                                        raid,
+                                                        udisks_block_get_mdraid_member,
+                                                        FALSE,    /* Retrieve all */
+                                                        FALSE);   /* Don't skip partitions */
 }
-
 /* ---------------------------------------------------------------------------------------------------- */
 
 static void
@@ -1604,14 +1559,15 @@ on_interface_proxy_properties_changed (GDBusObjectManagerClient   *manager,
   UDisksClient *client = UDISKS_CLIENT (user_data);
   UDisksClientClass *client_class = UDISKS_CLIENT_GET_CLASS (client);
 
-  GVariantIter *iter = NULL;
-  const gchar *property_name = NULL;
+  GVariantIter iter;
+  gchar *property_name = NULL;
 
   /* never emit the change signal for Job objects */
-  if (g_strcmp0 (g_dbus_proxy_get_interface_name (interface_proxy), "org.freedesktop.UDisks2.Drive.Job"))
+  if (g_strcmp0 (g_dbus_proxy_get_interface_name (interface_proxy), "org.freedesktop.UDisks2.Drive.Job") == 0)
     return;
 
-  while (g_variant_iter_next (iter, "{&sv}", &property_name, NULL))
+  g_variant_iter_init (&iter, changed_properties);
+  while (g_variant_iter_next (&iter, "{&sv}", &property_name, NULL))
     {
       if (! g_hash_table_contains (client_class->changed_blacklist, property_name))
         {
@@ -2533,9 +2489,12 @@ udisks_client_get_job_description (UDisksClient   *client,
       g_hash_table_insert (hash, (gpointer) "encrypted-modify",     (gpointer) C_("job", "Modifying Encrypted Device"));
       g_hash_table_insert (hash, (gpointer) "swapspace-start",      (gpointer) C_("job", "Starting Swap Device"));
       g_hash_table_insert (hash, (gpointer) "swapspace-stop",       (gpointer) C_("job", "Stopping Swap Device"));
+      g_hash_table_insert (hash, (gpointer) "filesystem-check",     (gpointer) C_("job", "Checking Filesystem"));
       g_hash_table_insert (hash, (gpointer) "filesystem-mount",     (gpointer) C_("job", "Mounting Filesystem"));
       g_hash_table_insert (hash, (gpointer) "filesystem-unmount",   (gpointer) C_("job", "Unmounting Filesystem"));
       g_hash_table_insert (hash, (gpointer) "filesystem-modify",    (gpointer) C_("job", "Modifying Filesystem"));
+      g_hash_table_insert (hash, (gpointer) "filesystem-repair",    (gpointer) C_("job", "Repairing Filesystem"));
+      g_hash_table_insert (hash, (gpointer) "filesystem-resize",    (gpointer) C_("job", "Resizing Filesystem"));
       g_hash_table_insert (hash, (gpointer) "format-erase",         (gpointer) C_("job", "Erasing Device"));
       g_hash_table_insert (hash, (gpointer) "format-mkfs",          (gpointer) C_("job", "Creating Filesystem"));
       g_hash_table_insert (hash, (gpointer) "loop-setup",           (gpointer) C_("job", "Setting Up Loop Device"));
